/**

Host

El agente PartyAgent es un esplndido anfitrin de fiestas. 
El anfitrin se registrar como anfitrin de la fiesta (party host). 


Cada vez que un agente es saludado por alguien siempre contestar con otro saludo.

Cuando un invitado est satisfecho se despedir del anfitrin con un saludo (adis, por 
ejemplo), ste le responder educadamente y el invitado se marchar de la 
fiesta.

**/

package actores.behaviours;

import java.io.*;
import java.util.*;
import jade.core.AID;
import jade.core.behaviours.*; 

import jade.core.Agent;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;



public class Host extends Agent
{
	private ArrayList<String> listaCanapes = new ArrayList<String>();
	private ArrayList<String> menu = new ArrayList<String>();

	protected void setup()
	{	
	
		DFAgentDescription listadoInvitados = new DFAgentDescription();
		listadoInvitados.setName( getAID() );
		ServiceDescription sd = new ServiceDescription();
		sd.setType("party-host");
		sd.setName( getLocalName() );
		listadoInvitados.addServices( sd );
		try 
		{
		  DFService.register(this, listadoInvitados);
		}
		catch (FIPAException fe) 
		{
		  fe.printStackTrace();
		}
		System.out.println("[Host] " + getLocalName() + " se viste con sus mejores galas.");
		
		doWait(5000);
		elegirMenu();
		
		
	}//fin setup()
	
	/**
		De un documento en el que hay multitud de platos, elegimos 20 canapes para la fiesta aleatoriamente
	**/
	
	
	private void elegirMenu()
	{
		try
		{
			String strLinea;
			
			FileInputStream fstream = new FileInputStream("Canapes.TXT");		
			DataInputStream entrada = new DataInputStream(fstream);		
			BufferedReader buffer = new BufferedReader(new InputStreamReader(entrada));
			
			
			while ((strLinea = buffer.readLine()) != null)  
			{
				
				listaCanapes.add(strLinea);
			}
				
			entrada.close();			
			
			
			System.out.println("\n\n["+getLocalName() +"]=============================Seleccion de Menu============================================");
			System.out.println("Los canapes elegidos han sido los siguientes:");
			
			//Elegimos 20 platos 
			
			while ( menu.size() != 20 )
			{
				Random randomGenerator = new Random();    
				int numero = randomGenerator.nextInt( listaCanapes.size() - 1 ); 
		
				if ( !menu.contains( listaCanapes.get( numero ) ) )
				{
					menu.add( listaCanapes.get( numero ) ) ;
				}
			}
			
			for ( int i=0 ; i< menu.size() ; i++ )
			{
				System.out.println( i+1 + " " + menu.get( i ) );
			}

			doWait(10000);
			addBehaviour( new enviarMenu() );
				
        }
		catch (Exception e)
		{ 
            System.err.println("Ha ocurrido un error: " + e.getMessage());
        }
		
	}//fin elegirMenu
	
	
	/** Una vez elegido el menu, se lo enviamos al coordinador para que este se lo comunique a los camareros **/
	private class enviarMenu extends OneShotBehaviour 
	{
			
		public void action()
		{
			System.out.println("\n\n===================================Enviando Menu===================================");
			
			  DFAgentDescription template = new DFAgentDescription();
			  ServiceDescription sd = new ServiceDescription();
			  sd.setType("party-coordinador");
			  template.addServices(sd);
			  
			  
			  try 
			  {
				DFAgentDescription[] result = DFService.search(myAgent, template); 
				System.out.println("[" + getLocalName() + "] Se han encontrado los siguientes coordinadores en la lista del Host:");
				AID[] elCoordinador = new AID[result.length];
				
				for (int i = 0; i < result.length; ++i)
				{
					elCoordinador[i] = result[i].getName();
					System.out.println("[" + getLocalName() +"] >> [" + elCoordinador[i].getLocalName() +"]");
					
					ACLMessage msgMenu = new ACLMessage( ACLMessage.INFORM ) ;
					
					msgMenu.addReceiver( elCoordinador[i] );
					msgMenu.setContent( "msgMenu" );
					msgMenu.setContentObject( menu ) ;
					msgMenu.setConversationId("msgMenu");
					msgMenu.setReplyWith("msgMenu"+System.currentTimeMillis());
					myAgent.send( msgMenu );
					
					doWait(3000);

									
				}
			  }
			  catch (FIPAException fe)
			  {
				fe.printStackTrace();
			  } 
			  
			  catch (Exception ex) 
			  { 
				ex.printStackTrace(); 
			  }
			  
			  System.out.println("\n\n======================================================================================");
			 
			 doWait(10000);
			 
			 addBehaviour( new esperarSaludo() );
		}
		
	}//fin enviarMenu

	
	/** Los invitados de nuestra cena iran saludando al padrino **/
	private class esperarSaludo extends CyclicBehaviour 
	{
	
		public void action() 
		{
			MessageTemplate mt = MessageTemplate.MatchPerformative(ACLMessage.PROPOSE);
			ACLMessage mensaje = myAgent.receive(mt);
		  
			if (mensaje != null)
			{
				try
				{
					doWait( 3000 ) ;
					// System.out.println("\n["+getLocalName() +"] he recibido algo de ...." + mensaje.getSender().getLocalName() + " " + mensaje.getContent() );
					 
					 
					// System.out.println("\n["+getLocalName() +"] >>>[" + mensaje.getSender().getLocalName() + "] Disfruta de la fiesta" ); 
					ACLMessage reply = mensaje.createReply();
					reply.setContent( "te devuelvo el saludo" );	
					reply.setPerformative( ACLMessage.REFUSE );
					// System.out.println("\n["+getLocalName() +"]>>>["+ mensaje.getSender().getLocalName() +"] HOLA MAJO!!!!" );
				  
				
					doWait(5000);
					myAgent.send(reply);
					
				}
				catch(Exception e)
				{
					e.printStackTrace(); 
				}
								
			}
			else
			{
				block();
			}
		}
	  
	}  // Fin esperarSaludo
	
	protected void takeDown()
	{
		System.out.println("\n[" + getLocalName() + "] abandona la fiesta.\n");
		
	}//fin takeDown()
}