/**

Invitado

Los agentes invitados se registrarn como invitados a la fiesta (party guest) 
y saludarn a todos las personas que estn en la fiesta (incluido el anfitrin).

Cada vez que un agente es saludado por alguien siempre contestar con 
otro saludo.

Cuando un invitado est satisfecho se despedir del anfitrin con un saludo
(adis, por ejemplo), ste le responder educadamente y el invitado se marchar de la fiesta.

Slo cuando hayan llegado todos los invitados a la fiesta (10 por ejemplo),
hayan comido y bebido suficiente empezarn a despedirse cada cierto tiempo. 

**/

package actores.behaviours;

import java.io.*;
import java.util.*;

import jade.core.Agent;
import jade.core.AID;
import jade.core.behaviours.*; 
import java.util.Random;

import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;

import java.util.*;



public class Invitado extends Agent
{

	private ArrayList<String> saludados = new ArrayList<String>();
	private ArrayList<String> pendientesSaludar = new ArrayList<String>();
	
	private AID[] enLaFiesta = new AID[11]; // coordinador + 10 invitados
	private AID[] elCoordinador ;
	private boolean avisadoCoordinador=false;
	private int saludarA = 0; 
	private String ultimo ;
	
	
	/**	
	Los agentes invitados se registrarn como invitados a la fiesta (party guest) 
	**/
	
	protected void setup()
	{
		DFAgentDescription listadoInvitados = new DFAgentDescription();
		listadoInvitados.setName( getAID() );
		ServiceDescription sd = new ServiceDescription();
		sd.setType("party-guest");
		sd.setName( getLocalName() );
		listadoInvitados.addServices( sd );
		try 
		{
		  DFService.register(this, listadoInvitados);
		}
		catch (FIPAException fe) 
		{
		  fe.printStackTrace();
		}
	
	
		System.out.println("[Invitado] " + getLocalName() + " esta disponible.");
		
		addBehaviour( new EsperarInvitacion() );
		
		addBehaviour( new Escuchar() );		
	
	}//fin setup()
	
	/** Esperamos a que nos envien la invitacion **/
	private class EsperarInvitacion extends CyclicBehaviour 
	{
	
		public void action() 
		{
			MessageTemplate mt = MessageTemplate.MatchPerformative(ACLMessage.REQUEST);
			ACLMessage mensaje = myAgent.receive(mt);
		  
			if (mensaje != null)
			{
				if( mensaje.getContent().equals("disparo"))
				{
					doDelete();
				}
				else
				{
					ACLMessage reply = mensaje.createReply();
				
					// Generamos un random [ 0-60 ] para saber cuando iremos a la fiesta
					
					Random randomGenerator = new Random();    
					// int esperar = randomGenerator.nextInt(60); //random 0-1...-58-59
					int esperar = randomGenerator.nextInt(60);
					// le sumo 11 segundos y lo multiplico por 1000, para saber los segundos que esperaremos en entrar
					
					esperar = ( esperar + 11 ) * 1000 ;
					
					
					reply.setPerformative( ACLMessage.CONFIRM );
					
					System.out.println("["+getLocalName() +"]>>>["+ mensaje.getSender().getLocalName() +"]Invitacion recibida. Nos vemos en la fiesta en ...." + esperar/1000 +" segundos.");
				
					myAgent.send(reply);
					
					doWait( esperar );				
					
					addBehaviour( new DeFiesta() );
				}				
			}
			else
			{
				block();
			}
		}	  
	}  // Fin EsperarInvitacion


	/** Intercambio de mensajes, conversacion con la gente que hay que saludar **/
	private class DeFiesta extends Behaviour
	{
		private MessageTemplate tratarRespuestas; // template para recibir lo que nos respondan 
		
		private int step = 0 ;
		
				
	
		public void action()
		{
					
			switch (step) 
			{
			
			//Ya estamos en la fiesta, y lo primero que hay que hacer es decirle al coordinador que nos presente.
			case 0:	 
			
				System.out.println("\n\n["+getLocalName() +"] Entra en la fiesta.");
				
				if ( !saludados.contains ( getLocalName() ) )
				{
					saludados.add( getLocalName() ) ;
				}
				
				// Enviamos las invitaciones al coordinador
			  
				DFAgentDescription template = new DFAgentDescription();
				ServiceDescription sd = new ServiceDescription();
				sd.setType("party-coordinador");
				template.addServices(sd);
			  
			  
				try 
				{
					DFAgentDescription[] result = DFService.search(myAgent, template); 
					// System.out.println("\n[" + getLocalName() + "] Se han encontrado los siguientes coordinadores:");
					elCoordinador = new AID[result.length];
				
					for (int i = 0; i < result.length; ++i)
					{
						elCoordinador[i] = result[i].getName();
						System.out.println("[" + getLocalName() +"] >> [" + elCoordinador[i].getLocalName() +"]");
						
						ACLMessage msgLlegada = new ACLMessage( ACLMessage.REQUEST ) ;
						
						msgLlegada.addReceiver( elCoordinador[i] );
						msgLlegada.setContent( "msgLlegada" );						
						msgLlegada.setConversationId("msgLlegada");
						msgLlegada.setReplyWith("msgLlegada"+System.currentTimeMillis());
						myAgent.send( msgLlegada );
						
						
						tratarRespuestas = MessageTemplate.and( MessageTemplate.MatchConversationId("msgLlegada")
										,MessageTemplate.MatchInReplyTo( msgLlegada.getReplyWith() ) );
						
						doWait(3000);									
					}
				}
				catch (FIPAException fe)
				{
					fe.printStackTrace();
				} 				  
				catch (Exception ex) 
				{ 
					ex.printStackTrace(); 
				}
			  
				
			 
doWait(3000);
				
				step++;
				break;
	
			
			case 1:
				try  
				{
					doWait(3000);	
					// System.out.println("\n"+getLocalName() +"===STEP 1   Analizando la respuesta del coordinador ===\n");
				
					ACLMessage respuesta = myAgent.receive( tratarRespuestas );
					
					if (respuesta != null && respuesta.getPerformative() == ACLMessage.AGREE ) 
					{
					
						enLaFiesta = (AID[])respuesta.getContentObject();	
						
						System.out.print("\n[" + getLocalName() +"] Tengo que saludar a: ");
						for( int i = 0 ; i < enLaFiesta.length  && enLaFiesta[i] != null 
										&& !enLaFiesta[i].getLocalName().equals( getLocalName() ) ; i++ )
						
						{
							System.out.print( " " +   enLaFiesta[i].getLocalName()  );
						}
						
						for( int i = 0 ; i < enLaFiesta.length  && enLaFiesta[i] != null 
										&& !enLaFiesta[i].getLocalName().equals( getLocalName() ) ; i++ )
						{
															
							ACLMessage msgNoHuyas = new ACLMessage( ACLMessage.DISCONFIRM ) ;
						
							// System.out.println("\n[" + getLocalName() + "]"  +  "]" + " >>> ["+ enLaFiesta[saludarA].getLocalName() +"] Ciao " + enLaFiesta[saludarA].getLocalName() + ", tutto va bene, grazie") ;
						
							msgNoHuyas.addReceiver( enLaFiesta[ i ] );
							msgNoHuyas.setContent( "msgNoHuyas" );						
							msgNoHuyas.setConversationId("msgNoHuyas");
							msgNoHuyas.setReplyWith("msgNoHuyas"+System.currentTimeMillis());
							myAgent.send( msgNoHuyas );							
								
						}
						System.out.println();
						
					}
					
					step = 2 ;

				}
				catch(Exception e)
				{
					e.printStackTrace(); 
				}
				
				break;
				
			case 2:
				try
				{
					doWait(3000);	
					
					if ( enLaFiesta[ saludarA] != null 	&& !enLaFiesta[saludarA].getLocalName().equals( getLocalName() ) )
					{
						// System.out.println("\n"+getLocalName() +"===STEP 2   Envio Saludo a una persona de la fiesta ===\n");		
						
						ACLMessage msgHola = new ACLMessage( ACLMessage.PROPOSE ) ;
						
						// System.out.println("\n[" + getLocalName() + "]"  +  "]" + " >>> ["+ enLaFiesta[saludarA].getLocalName() +"] Ciao bambino.");
						
						ultimo = enLaFiesta[ saludarA ].getLocalName();
						msgHola.addReceiver( enLaFiesta[ saludarA ] );
						msgHola.setContent( "HOLA CARACOLA ENVIO CASE 2" );						
						msgHola.setConversationId("HOLA CARACOLA");
						msgHola.setReplyWith("HOLA CARACOLA"+System.currentTimeMillis());
						myAgent.send( msgHola );
						
						// System.out.println("ACABO DE ENVIAR EL MENSAJE EN CASE 2 : " + msgHola );
						tratarRespuestas = MessageTemplate.and( MessageTemplate.MatchConversationId("msgHola")
										,MessageTemplate.MatchInReplyTo( msgHola.getReplyWith() ) );
						
doWait(3000);	
						step = 3;
						
					}
					else
					{
						step = 4 ;
					}
					 
					
					
				}
				catch(Exception e)
				{
					e.printStackTrace(); 
				}
				break;
			
			case 3:
				doWait(3000);	
				// System.out.println("\n"+getLocalName() +"===STEP 3  La otra persona nos devuelve el saludo ===\n");
					
				ACLMessage respuesta  = myAgent.receive( tratarRespuestas );
				// System.out.println( respuesta );
				System.out.println( "<<< [" + getLocalName() +"] y [" + ultimo + "] se han saludado mutuamente >>>");
				saludarA++ ;					
				step = 4 ;
				
				if ( respuesta == null ) //&& (respuesta.getPerformative() == ACLMessage.REFUSE) )
				{
					// System.out.println( "<<< [" + getLocalName() +"] y [" + respuesta.getSender().getLocalName() + "] se han saludado mutuamente >>>");
					
					// saludados.add( respuesta.getSender().getLocalName() ) ;
					saludarA++ ;
					
					step = 4 ;
				}
				else 
				{
					block();
				}
				break;
				
			case 4:
				// doWait(3000);	
				if ( saludarA == enLaFiesta.length - 1 )
				{
					// System.out.println("\n"+getLocalName() + "===STEP 4  No hay que saludar a mas gente ===\n");
					System.out.println("\n********[" + getLocalName() + "] ya ha saludado a todo el mundo, y una vez saciado, intenta abandonar la fiesta pero el coordinador de la fiesta le pide que se quede hasta el final.********\n");
					addBehaviour( new EsperarRetrasados() ) ;
					
					// System.out.println("[" + getLocalName() +"] >> [" + elCoordinador[0].getLocalName() +"]");
						
					ACLMessage msgAdios = new ACLMessage( ACLMessage.INFORM ) ;
						
					msgAdios.addReceiver( elCoordinador[0] );
					msgAdios.setContent( "adios" );						
					msgAdios.setConversationId("adios");
					msgAdios.setReplyWith("adios"+System.currentTimeMillis());
					myAgent.send( msgAdios );
					
doWait(3000);					
					step = 5 ;
				}
				else if ( enLaFiesta[saludarA] == null || enLaFiesta[saludarA].getLocalName().equals( getLocalName() ) )
				{
					saludarA++;
					step = 2 ;
				}
				else
				{
					// System.out.println("\n"+getLocalName() +"===STEP 4  Hay que saludar a mas gente ===\n");
					step = 2 ;
				}
				
				
				break;
			
			}
		}
	
		public boolean done() 
		{
			return  ( step == 5 );			
		}	
	}//fin DeFiesta
	
	 /** recibo mensajes de otras personas  **/
	private class Escuchar extends CyclicBehaviour 
	{
	
		public void action() 
		{
			MessageTemplate mt = MessageTemplate.MatchPerformative(ACLMessage.PROPOSE);
			ACLMessage mensaje = myAgent.receive(mt);
		  
			if (mensaje != null)
			{
				try
				{
			
					// doWait( 3000 ) ;
					// System.out.println("\n["+getLocalName() +"] he recibido algo de ...." + mensaje.getSender().getLocalName() + " " + mensaje.getContent() );
					
					ACLMessage reply = mensaje.createReply();	
					reply.setPerformative( ACLMessage.REFUSE );
					// System.out.println("\n["+getLocalName() +"]>>>["+ mensaje.getSender().getLocalName() +"] Se tutto va bene, tornero il prossimo anno a mostrarvi molto di piu. se non va tutto bene, saro in prigione come Wen Ho Lee" );
					reply.setContent("hola guapeton");
					pendientesSaludar.remove ( mensaje.getSender().getLocalName() ) ;
					saludados.add ( mensaje.getSender().getLocalName() ) ;
					
				  				
doWait(3000);
					myAgent.send(reply);
					
					// System.out.println( "******************************" + getLocalName() + " esperando " +  pendientesSaludar.size() + " saludos.");
										
				}
				catch(Exception e)
				{
					e.printStackTrace(); 
				}
								
			}
			else
			{
				block();
			}
		}
	  
	}  // Fin Escuchar
	
	/** Metodo para los casos que nos queramos ir de la fiesta y justo en ese momento entre alguien , lo aadimos a una lista temporal para saludarles**/
	private class EsperarRetrasados extends CyclicBehaviour 
	{
	
		public void action() 
		{
			MessageTemplate mt = MessageTemplate.MatchPerformative(ACLMessage.DISCONFIRM);
			ACLMessage mensaje = myAgent.receive(mt);
		  
			if (mensaje != null)
			{	
				// System.out.println("\n"+ getLocalName() + "<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<HASTA QUE NO ME SALUDE  " + mensaje.getSender().getLocalName()  + " NO ME PUEDO IR.>>>>>>>>>>>>>>>>>>>>>>>>>>");
				pendientesSaludar.add ( mensaje.getSender().getLocalName() ) ;
			}
			else
			{
				block();
			}
		}
		
	  
	}  // Fin Esperar Retrasados
	
	protected void takeDown()
	{
		int esperar;
		
		//El tiroteo dura 30 segundos tiempo en el que deberemos huir o quedarnos
		Random randomGenerator = new Random();    		
		esperar = randomGenerator.nextInt(30);
		doWait( ( esperar + 15 )  * 1000 ) ;
		
		
		if ( esperar % 2 == 0 )
		{
			System.out.println("Con su ultimo aliento," + getLocalName() + "se despide de su padrino.... ciao....pa...p......pa.");
			System.out.println("[DonVito]>>>["+getLocalName()+"] Tutti... fruti , bambino ");
			
		}
		else
		{
			System.out.println( "[" + getLocalName() + "]>>>[DonVito] huye conmigo por la puerta trasera y nos vengaremos.");
			System.out.println("[DonVito]>>>["+getLocalName()+"] Hablas de venganza. La venganza no devuelve hijos...");
		}
		
	}//fin takeDown()
	
}//Fin Clase Invitado